import sys


def resource_generate_samples_queries(period):
    procedure_name = "get_" + period.lower() + "_samples"

    howmany = 24
    GOBACK = "HOUR"
    COEF = 1
    if period.lower() == "daily":
        howmany = 24
        GOBACK = "HOUR"
        COEF = 1
    elif period.lower() == "weekly":
        howmany = 7 * 4
        GOBACK = "HOUR"
        COEF = 6
    elif period.lower() == "monthly":
        howmany = 31
        GOBACK = "DAY"
        COEF = 1

    print("delimiter //")
    print("DROP PROCEDURE IF EXISTS %s;" % procedure_name)
    print("CREATE PROCEDURE %s (" % procedure_name)
    print("    IN id INT,")
    for i in range(howmany):
        print("    OUT entry%d FLOAT," % i)
    print("    OUT start_hour FLOAT")
    print(" )")

    print("BEGIN")
    if period.lower() == "monthly":
        print("    select DAY(now()) INTO start_hour;")
    else:
        print("    select HOUR(now()) INTO start_hour;")

    for i in range(howmany):
        print(
            "    if EXISTS(SELECT ResponseTime FROM PING WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL %d %s) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL %d %s) AND Resourceid = id)"
            % (COEF * (howmany - i), GOBACK, COEF * (howmany - 1 - i), GOBACK))
        print(
            "        then SELECT ResponseTime INTO entry%d FROM PING WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL %d %s) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL %d %s) AND Resourceid = id limit 1;"
            % (i, COEF * (howmany - i), GOBACK, COEF *
               (howmany - 1 - i), GOBACK))
        print("        else SET entry%d := 0;" % i)
        print("    end if;")

    print("END//")
    print("delimiter ;")


if __name__ == "__main__":
    if len(sys.argv) <= 1:
        print("Usage: python3 resource_generate_samples_queries.py <period>")
        sys.exit(-1)
    resource_generate_samples_queries(sys.argv[1])



import matplotlib.pyplot as plt
import numpy as np
import sys

from rComplexity.plotting.constants import pool_plotting_points_symbols

CONFIG = {
    1: "Naive Matrix Multiplication",
    2: "Cache-friendly loop ordering",
    3: "Blocked Matrix Multiplication",
}


def plot_points_from_file(filename, matrixCols):
    # Load Matrix
    matrix = np.loadtxt(filename, usecols=range(matrixCols + 1))

    for i in range(1, matrixCols + 1):
        name = CONFIG.get(i, str(i))
        plt.plot(
            matrix[:, 0],
            matrix[:, i],
            next(pool_plotting_points_symbols),
            label="Sample: " + name,
        )

    plt.legend()
    plt.show()


if __name__ == "__main__":
    if len(sys.argv) <= 2:
        print("Usage: python3 plot_points.py <file_name> <matrixCols>")
        sys.exit(-1)

    plot_points_from_file(sys.argv[1], int(sys.argv[2]))



import pymysql
from flask import Flask
from flask import jsonify
from flask import request
from flask_cors import CORS
from flaskext.mysql import MySQL

from WebMonitoring.API.add.add_resource import add_resource_wrapper
from WebMonitoring.API.add.add_website import add_website_wrapper
from WebMonitoring.API.constants import PERIOD
from WebMonitoring.API.delete.delete_resource import delete_resource_wrapper
from WebMonitoring.API.delete.delete_website import delete_website_wrapper
from WebMonitoring.API.resources.efficiency_metrics import (
    get_results_resource_get_efficiency, )
from WebMonitoring.API.resources.metrics_renderer import render_dict
from WebMonitoring.API.resources.samples_size_resource import (
    resources_get_samples_size_daily,
    resources_get_samples_size_monthly,
    resources_get_samples_size_weekly,
)
from WebMonitoring.API.resources.samples_time_resource import (
    resources_get_samples_time_daily,
    resources_get_samples_time_monthly,
    resources_get_samples_time_weekly,
)
from WebMonitoring.API.resources.size_metrics import get_results_resource_get_size
from WebMonitoring.API.resources.time_metrics import get_results_resource_get_time
from WebMonitoring.API.settings import (
    MYSQL_DATABASE_HOST,
    MYSQL_DATABASE_USER,
    MYSQL_DATABASE_PASSWORD,
    MYSQL_DATABASE_DB,
)
from WebMonitoring.API.websites.samples_size_websites import (
    websites_get_samples_size_daily,
    websites_get_samples_size_weekly,
    websites_get_samples_size_monthly,
)
from WebMonitoring.API.websites.samples_time_websites import (
    websites_get_samples_time_daily,
    websites_get_samples_time_weekly,
    websites_get_samples_time_monthly,
)

app = Flask(__name__)
CORS(app)

mysql = MySQL()

# MySQL configurations
app.config["MYSQL_DATABASE_USER"] = MYSQL_DATABASE_USER
app.config["MYSQL_DATABASE_PASSWORD"] = MYSQL_DATABASE_PASSWORD
app.config["MYSQL_DATABASE_DB"] = MYSQL_DATABASE_DB
app.config["MYSQL_DATABASE_HOST"] = MYSQL_DATABASE_HOST

# Workaround for KeyError: MYSQL_DATABASE_SOCKET
app.config["MYSQL_DATABASE_SOCKET"] = None

mysql.init_app(app)


@app.route("/")
def index():
    return "Hello, world!"


@app.route("/availableResources/<username>")
def available_resources(username):
    try:
        conn = mysql.connect()
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute(
            "select r.Resourceid id ,r.Resourceid id_resource, r.ResourceName name, r.Command command, r.FirstAdded firstadded, resource_get_availability(r.Resourceid) status from RESOURCE r, USERS u where u.Userid = r.Userid AND u.Username='%s'"
            % (username))
        rows = cursor.fetchall()

        resp = []
        count_id = 0
        for row in rows:
            resp.append(row)
            resp[-1]["id"] = count_id
            count_id = count_id + 1

        resp = jsonify(rows)
        resp.status_code = 200
        return resp
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.route("/availableWebsites/<username>")
def available_websites(username):
    try:
        conn = mysql.connect()
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute(
            "select r.Websiteid id ,r.Websiteid id_resource, r.WebsiteName name, r.WebsiteUrl command, r.FirstAdded firstadded, \"WORKING\" status from WEBSITES r, USERS u where u.Userid = r.Userid AND u.Username='%s'"
            % (username))
        rows = cursor.fetchall()

        resp = []
        count_id = 0
        for row in rows:
            resp.append(row)
            resp[-1]["id"] = count_id
            count_id = count_id + 1

        resp = jsonify(rows)
        resp.status_code = 200
        return resp
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.route("/resources/metrics/<resource_name>")
def resources_metrics(resource_name):
    (
        result_args_get_time,
        result_args_old_get_time,
        list_sample_time,
    ) = get_results_resource_get_time(mysql, resource_name)
    (
        result_args_get_size,
        result_args_old_get_size,
        list_sample_size,
    ) = get_results_resource_get_size(mysql, resource_name)
    (
        result_args_get_efficiency,
        result_args_old_get_efficiency,
        list_sample_efficiency,
    ) = get_results_resource_get_efficiency(
        result_args_get_time,
        result_args_old_get_time,
        list_sample_time,
        result_args_get_size,
        result_args_old_get_size,
        list_sample_size,
    )
    resp = jsonify([
        render_dict(
            result_args_get_time,
            result_args_old_get_time,
            list_sample_time,
            "Request Time",
            "primary",
            roundDecimal=2,
        ),
        render_dict(
            result_args_get_size,
            result_args_old_get_size,
            list_sample_size,
            "Response Size",
            "warning",
            roundDecimal=0,
        ),
        render_dict(
            result_args_get_efficiency,
            result_args_old_get_efficiency,
            list_sample_efficiency,
            "Efficiency",
            "secondary",
            roundDecimal=0,
        ),
    ])
    resp.status_code = 200
    return resp


@app.route("/resources/samples/time/<resource_id>/<period>")
def resources_get_samples_time(resource_id, period):
    samples = {}
    if period.lower() == PERIOD.DAILY:
        samples = resources_get_samples_time_daily(mysql, resource_id)
    if period.lower() == PERIOD.WEEKLY:
        samples = resources_get_samples_time_weekly(mysql, resource_id)
    if period.lower() == PERIOD.MONTHLY:
        samples = resources_get_samples_time_monthly(mysql, resource_id)

    resp = jsonify(samples)
    resp.status_code = 200
    return resp


@app.route("/websites/samples/time/<website_id>/<period>")
def websites_get_samples_time(website_id, period):
    samples = {}
    if period.lower() == PERIOD.DAILY:
        samples = websites_get_samples_time_daily(mysql, website_id)
    if period.lower() == PERIOD.WEEKLY:
        samples = websites_get_samples_time_weekly(mysql, website_id)
    if period.lower() == PERIOD.MONTHLY:
        samples = websites_get_samples_time_monthly(mysql, website_id)

    resp = jsonify(samples)
    resp.status_code = 200
    return resp


@app.route("/resources/samples/size/<resource_id>/<period>")
def resources_get_samples_size(resource_id, period):
    samples = {}
    if period.lower() == PERIOD.DAILY:
        samples = resources_get_samples_size_daily(mysql, resource_id)
    if period.lower() == PERIOD.WEEKLY:
        samples = resources_get_samples_size_weekly(mysql, resource_id)
    if period.lower() == PERIOD.MONTHLY:
        samples = resources_get_samples_size_monthly(mysql, resource_id)

    resp = jsonify(samples)
    resp.status_code = 200
    return resp


@app.route("/websites/samples/size/<website_id>/<period>")
def websites_get_samples_size(website_id, period):
    samples = {}
    if period.lower() == PERIOD.DAILY:
        samples = websites_get_samples_size_daily(mysql, website_id)
    if period.lower() == PERIOD.WEEKLY:
        samples = websites_get_samples_size_weekly(mysql, website_id)
    if period.lower() == PERIOD.MONTHLY:
        samples = websites_get_samples_size_monthly(mysql, website_id)

    resp = jsonify(samples)
    resp.status_code = 200
    return resp


@app.route("/resources/statistics")
def resources_statistics():
    try:
        conn = mysql.connect()

        # Time
        cursor_all_time = conn.cursor(pymysql.cursors.DictCursor)
        cursor_all_time.execute(
            "select resource_statistic_requests_time() requests_all, "
            "resource_statistic_time() time_all, "
            "resource_statistic_average_time() average_time_all, "
            "resource_statistic_standard_deviation_time() standard_deviation_all "
            "from DUAL")
        fetch = cursor_all_time.fetchone()
        (requests_all) = fetch

        cursor_24_time = conn.cursor(pymysql.cursors.DictCursor)
        cursor_24_time.execute(
            "select resource_statistic_requests_time_24() requests_24, "
            "resource_statistic_time_24() time_24, "
            "resource_statistic_average_time_24() average_time_24, "
            "resource_statistic_standard_deviation_time_24() standard_deviation_24 "
            "from DUAL")
        fetch = cursor_24_time.fetchone()
        (requests_24) = fetch

        # Size
        cursor_all_size = conn.cursor(pymysql.cursors.DictCursor)
        cursor_all_size.execute(
            "select "
            "resource_statistic_size() size_all, "
            "resource_statistic_average_size() average_size_all, "
            "resource_statistic_standard_deviation_size() standard_deviation_all "
            "from DUAL")
        fetch = cursor_all_size.fetchone()
        (requests_all_size) = fetch

        cursor_24_size = conn.cursor(pymysql.cursors.DictCursor)
        cursor_24_size.execute(
            "select "
            "resource_statistic_size_24() size_24, "
            "resource_statistic_average_size_24() average_size_24, "
            "resource_statistic_standard_deviation_size_24() standard_deviation_24 "
            "from DUAL")
        fetch = cursor_24_size.fetchone()
        (requests_24_size) = fetch

        statistics = {
            "requests_24": round(requests_24["requests_24"], 2),
            "requests_all": round(requests_all["requests_all"], 2),
            "time_24": round(requests_24["time_24"], 2),
            "time_all": round(requests_all["time_all"], 2),
            "average_time_24": round(requests_24["average_time_24"], 2),
            "average_time_all": round(requests_all["average_time_all"], 2),
            "sd_time_24": round(requests_24["standard_deviation_24"], 2),
            "sd_time_all": round(requests_all["standard_deviation_all"], 2),
            "size_24": round(requests_24_size["size_24"], 2),
            "size_all": round(requests_all_size["size_all"], 2),
            "average_size_24": round(requests_24_size["average_size_24"], 2),
            "average_size_all": round(requests_all_size["average_size_all"],
                                      2),
            "sd_size_24": round(requests_24_size["standard_deviation_24"], 2),
            "sd_size_all": round(requests_all_size["standard_deviation_all"],
                                 2),
        }
        resp = jsonify(statistics)
        resp.status_code = 200
        return resp
    except Exception as e:
        print(e)
    finally:
        try:
            cursor_all_time.close()
            cursor_24_time.close()
            cursor_all_size.close()
            cursor_24_size.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.route("/websites/statistics")
def websites_statistics():
    try:
        conn = mysql.connect()

        # Time
        cursor_all_time = conn.cursor(pymysql.cursors.DictCursor)
        cursor_all_time.execute(
            "select FLOOR(SUM(TotalTime)) time_all, count(*) requests_all, AVG(TotalTime) average_time_all, STD(TotalTime) standard_deviation_all from WEBSITES_METRICS"
        )
        fetch = cursor_all_time.fetchone()
        (requests_all) = fetch

        cursor_24_time = conn.cursor(pymysql.cursors.DictCursor)
        cursor_24_time.execute(
            "select FLOOR(SUM(TotalTime)) time_24, count(*) requests_24, AVG(TotalTime) average_time_24, STD(TotalTime) standard_deviation_24 from WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 24 HOUR)"
        )
        fetch = cursor_24_time.fetchone()
        (requests_24) = fetch

        # Size
        cursor_all_size = conn.cursor(pymysql.cursors.DictCursor)
        cursor_all_size.execute(
            "select SUM(bodySize) size_all, AVG(bodySize) average_size_all, STD(bodySize) standard_deviation_all from REQUESTS"
        )
        fetch = cursor_all_size.fetchone()
        (requests_all_size) = fetch

        cursor_24_size = conn.cursor(pymysql.cursors.DictCursor)
        cursor_24_size.execute(
            "select SUM(bodySize) size_24, AVG(bodySize) average_size_24, STD(bodySize) standard_deviation_24 from REQUESTS WHERE startedDateTime >= DATE_SUB(NOW(), INTERVAL 24 HOUR)"
        )
        fetch = cursor_24_size.fetchone()
        (requests_24_size) = fetch

        statistics = {
            "requests_24": round(requests_24["requests_24"], 2),
            "requests_all": round(requests_all["requests_all"], 2),
            "time_24": round(requests_24["time_24"], 2),
            "time_all": round(requests_all["time_all"], 2),
            "average_time_24": round(requests_24["average_time_24"], 2),
            "average_time_all": round(requests_all["average_time_all"], 2),
            "sd_time_24": round(requests_24["standard_deviation_24"], 2),
            "sd_time_all": round(requests_all["standard_deviation_all"], 2),
            "size_24": round(requests_24_size["size_24"], 2),
            "size_all": round(requests_all_size["size_all"], 2),
            "average_size_24": round(requests_24_size["average_size_24"], 2),
            "average_size_all": round(requests_all_size["average_size_all"],
                                      2),
            "sd_size_24": round(requests_24_size["standard_deviation_24"], 2),
            "sd_size_all": round(requests_all_size["standard_deviation_all"],
                                 2),
        }
        resp = jsonify(statistics)
        resp.status_code = 200
        return resp
    except Exception as e:
        print(e)
    finally:
        try:
            cursor_all_time.close()
            cursor_24_time.close()
            cursor_all_size.close()
            cursor_24_size.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.route("/addresource", methods=["POST"])
def add_resource():
    return add_resource_wrapper(mysql, request)


@app.route("/addwebsite", methods=["POST"])
def add_website():
    return add_website_wrapper(mysql, request)


@app.route("/deleteresource", methods=["POST"])
def delete_resource():
    return delete_resource_wrapper(mysql, request)


@app.route("/deletewebsite", methods=["POST"])
def delete_website():
    return delete_website_wrapper(mysql, request)


@app.errorhandler(404)
def not_found(error=None):
    message = {"status": 404, "message": "Not Found: " + request.url}
    resp = jsonify(message)
    resp.status_code = 404

    return resp


if __name__ == "__main__":
    app.run(host="0.0.0.0", debug=True)



def polynomial_to_LaTeX(p):
    """ Small function to print nicely the polynomial p as we write it in maths, in LaTeX code."""
    coefs = p.coef  # List of coefficient, sorted by increasing degrees
    res = ""  # The resulting string
    for i, a in enumerate(coefs):
        if int(a) == a:  # Remove the trailing .0
            a = int(a)
        if i == 0:  # First coefficient, no need for X
            if a > 0:
                res += "{a} + ".format(a=("%.2E" % a))
            elif a < 0:  # Negative a is printed like (a)
                res += "({a}) + ".format(a=("%.2E" % a))
            # a = 0 is not displayed
        elif i == 1:  # Second coefficient, only X and not X**i
            if a == 1:  # a = 1 does not need to be displayed
                res += "X + "
            elif a > 0:
                res += "{a} \;X + ".format(a=("%.2E" % a))
            elif a < 0:
                res += "({a}) \;X + ".format(a=("%.2E" % a))
        else:
            if a == 1:
                # A special care needs to be addressed to put the exponent in {..} in LaTeX
                res += "X^{i} + ".format(i="{%d}" % i)
            elif a > 0:
                res += "{a} \;X^{i} + ".format(a=("%.2E" % a), i="{%d}" % i)
            elif a < 0:
                res += "({a}) \;X^{i} + ".format(a=("%.2E" % a), i="{%d}" % i)
    return "$" + res[:-3] + "$" if res else ""



from WebMonitoring.API.resources.last_seven_entries_metrics import (
    get_last_seven_entries_time, )


def get_results_resource_get_time(mysql, resource_name):
    result_args_get_time = []
    result_args_old_get_time = []
    list_sample = []

    try:
        conn = mysql.connect()
        cursor = conn.cursor()
        """
        Compatibility warning: PEP-249 specifies that any modified
        parameters must be returned. This is currently impossible
        as they are only available by storing them in a server
        variable and then retrieved by a query. Since stored
        procedures return zero or more result sets, there is no
        reliable way to get at OUT or INOUT parameters via callproc.
        The server variables are named @_procname_n, where procname
        is the parameter above and n is the position of the parameter
        (from zero). Once all result sets generated by the procedure
        have been fetched, you can issue a SELECT @_procname_0, ...
        query using .execute() to get any OUT or INOUT values.
        """
        args = [resource_name, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        cursor.callproc("resource_get_time", args)
        cursor.execute("SELECT @_resource_get_time_0, "
                       "@_resource_get_time_1, "
                       "@_resource_get_time_2, "
                       "@_resource_get_time_3, "
                       "@_resource_get_time_4, "
                       "@_resource_get_time_5, "
                       "@_resource_get_time_6, "
                       "@_resource_get_time_7, "
                       "@_resource_get_time_8, "
                       "@_resource_get_time_9, "
                       "@_resource_get_time_10, "
                       "@_resource_get_time_11, "
                       "@_resource_get_time_12 ")
        result_args_get_time = cursor.fetchone()

        cursor.callproc("resource_get_old_time", args)
        cursor.execute("SELECT @_resource_get_old_time_0, "
                       "@_resource_get_old_time_1, "
                       "@_resource_get_old_time_2, "
                       "@_resource_get_old_time_3, "
                       "@_resource_get_old_time_4, "
                       "@_resource_get_old_time_5, "
                       "@_resource_get_old_time_6, "
                       "@_resource_get_old_time_7, "
                       "@_resource_get_old_time_8, "
                       "@_resource_get_old_time_9, "
                       "@_resource_get_old_time_10, "
                       "@_resource_get_old_time_11, "
                       "@_resource_get_old_time_12 ")
        result_args_old_get_time = cursor.fetchone()

        list_sample = []
        samples = get_last_seven_entries_time(mysql, resource_name)
        for sample in samples:
            list_sample.append(int(1000 * sample["ResponseTime"] + 1))
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return (None, None, None)
        return (result_args_get_time, result_args_old_get_time, list_sample)



import matplotlib.pyplot as plt
import sys

from rafComputing.ML.CustomSettings.settings import MAX_ITER, AUTO_DRIVER_DEFAULT_OUTPUT_NAME
from rafComputing.ML.features.feature_types import POWER_FEATURE_TYPE
from rafComputing.ML.snippets.MLdriver_more_graphs import LinearRegressionTrainingMoreGraphs


def get_feature_val_from_path(path):
    # Regular
    feature_val = 3
    if "28" in path:
        # Strassen
        feature_val = 2.8
    return feature_val


if __name__ == "__main__":
    if len(sys.argv) <= 1:
        print(
            "Usage v1: python3 rafComputing/ML/snippets/demo/strassen/ML_strassen_more_graphs.py <file_name1> <file_name2> <file_name3> ... "
        )
        print(
            "Usage (e.g.): python3 rafComputing/ML/snippets/demo/strassen/ML_strassen_more_graphs.py rComplexity/samples/matrix_multiplication/results/sprmcrogpu-wn13/result_n3_20200417113226 rComplexity/samples/matrix_multiplication/results/sprmcrogpu-wn13/result_n28_20200417113226"
        )
        sys.exit(-1)

    counter = 0
    for path in sys.argv:
        if path == sys.argv[0]:
            continue
        LinearRegressionTrainingMoreGraphs(
            path=path,
            alpha=1e-19,
            n_iterations=MAX_ITER,
            internal_counter=counter,
            feature_type=POWER_FEATURE_TYPE,
            feature_val=get_feature_val_from_path(path))
        counter = counter + 1

    plt.legend([
        "Regression line: Cache-friendly loop ordering",
        "Regression line: Strassen algorithm",
        "Train data: Cache-friendly loop ordering",
        "Test data:  Cache-friendly loop ordering",
        "Train data: Strassen algorithm",
        "Test data:  Strassen algorithm",
    ])

    # Return figure
    plt.savefig(AUTO_DRIVER_DEFAULT_OUTPUT_NAME)
    plt.close()



import subprocess

from WebMonitoring.configs.constants import DockerResults
from WebMonitoring.configs.settings import SPEEDPROFILE_LOCATION


class SpeedprofileClient:
    """
    Client for https://github.com/parasdahal/speedprofile


    ACTUAL:
    SpeedprofileClient executes entrypoint.sh script

    DEPRECATED:
    SpeedprofileClient class receives a URL and runs a new docker
    with https://github.com/parasdahal/speedprofile image
    and helps capture HTTP Archive (HAR) and additional performance data
    using Navigation Timing API from either Chrome or Firefox headlessly.
    """

    # Default timeout for docker run command
    TIMEOUT = 90

    def __init__(self, url):
        self.url = url
        self.status = DockerResults.STARTED

    def run(self):
        # Set current status to FAILED
        self.status = DockerResults.FAILED

        # Deprecated:
        #    os.system(
        #    "docker run -v $(pwd)/output:/output ccarpita/speedprofile-chrome \"$@\" %s"
        #    % self.url)
        try:
            subprocess.call("%s %s" % (SPEEDPROFILE_LOCATION, self.url),
                            shell=True,
                            timeout=90)
            self.status = DockerResults.SUCCESS
        except subprocess.TimeoutExpired:
            self.status = DockerResults.FAILED
        finally:
            pass



__author__ = ["[Brandon Amos](http://bamos.github.io)"]
__date__ = "2014.02.14"
"""
`rank-writing.py` ranks the writing quality of my
blog's Markdown posts and my project's Markdown README files.

The following programs should be on your `PATH`:
+ [aspell](http://aspell.net/)
+ [write-good](https://github.com/btford/write-good)
+ [diction](https://www.gnu.org/software/diction/)


```
$ rank-writing.py *.md

=== 2013-05-03-scraping-tables-python.md ===
Total: 53
├── aspell: 34
├── diction: 0
└── write-good: 19

...

=== 2013-04-16-pdf-from-plaintext.md ===
Total: 0
├── aspell: 0
├── diction: 0
└── write-good: 0
```
"""

import argparse
import re
import shutil
from subprocess import Popen, PIPE

import sys

parser = argparse.ArgumentParser()
parser.add_argument("--aspell", action="store_true")
parser.add_argument("--diction", action="store_true")
parser.add_argument("--write-good", action="store_true")
parser.add_argument("files", type=str, nargs="+")
args = parser.parse_args()

# If no arguments are provided, show everything.
if not (args.aspell or args.diction or args.write_good):
    args.aspell = args.diction = args.write_good = True

programs = []
if args.aspell:
    programs.append("aspell")
if args.diction:
    programs.append("diction")
if args.write_good:
    programs.append("write-good")

for program in programs:
    if not shutil.which(program):
        print("Error: '{}' not found on PATH. Please install.".format(program))
        sys.exit(-1)


def call(cmd, in_f=None):
    if in_f:
        with open(in_f, "r") as f:
            p = Popen(cmd, stdout=PIPE, stderr=PIPE, stdin=f)
    else:
        p = Popen(cmd, stdout=PIPE, stderr=PIPE)
    out = p.communicate()[0].decode()
    if p.returncode != 0:
        raise Exception("Error running {}".format(cmd))
    return out


def getNumSuggestions(program, f_name):
    if program == "write-good":
        out = call(["write-good", f_name])
        return out.count("-------------")
    elif program == "diction":
        out = call(["diction", "--suggest", f_name])
        r = re.search("(\S*) phrases? in (\S*) sentences? found.", out)
        if r:
            if r.group(1) == "No":
                return 0
            else:
                return int(r.group(1))
    elif program == "aspell":
        out = call(["aspell", "list"], in_f=f_name)
        return len(out.split())
    else:
        raise Exception("Unrecognized program: {}".format(program))


def getSuggestions(f_name):
    suggestions = list(map(lambda x: getNumSuggestions(x, f_name), programs))
    return (sum(suggestions), suggestions, f_name)


for tup in sorted(map(getSuggestions, args.files), reverse=True):
    print("\n=== {} ===".format(tup[2]))
    print("    Total: {}".format(tup[0]))
    div = ["├──"] * (len(programs) - 1) + ["└──"]
    for program_output in list(zip(div, programs, tup[1])):
        print("    {} {}: {}".format(*program_output))



SAMPLE_DOCKER_COMPOSE = """version: "3.3"

# docker stack deploy -c docker_config/docker-compose.yaml rafMetrics
services:
  # docker build -f mysql_config/Dockerfile -t raresraf/rafmetricsmysql:latest .
  mysql:
    image: raresraf/rafmetricsmysql:latest
    environment:
      DOCKER_COMPOSE_BUILD: "true"
    ports:
      - "3306:3306"
    # expose:
    #  - '3306'
    networks:
      - backend
    volumes:
      - db-volume:/var/lib/mysql
    deploy:
      replicas: 1
      placement:
        constraints: [node.role == manager]

  # docker build -f WebMonitoring/DockerfileResource -t raresraf/resourcemonitor:latest .
  resourcemonitor:
    image: raresraf/resourcemonitor:latest
    depends_on:
      - mysql
    environment:
      DOCKER_COMPOSE_BUILD: "true"
    networks:
      - backend
    deploy:
      replicas: 1
      placement:
        constraints: [node.role == worker]

  # docker build -f WebMonitoring/DockerfileWebsite -t raresraf/websitemonitor:latest .
  websitemonitor:
    image: raresraf/websitemonitor:latest
    depends_on:
      - mysql
    environment:
      DOCKER_COMPOSE_BUILD: "true"
    networks:
      - backend
    deploy:
      replicas: 1
      placement:
        constraints: [node.role == worker]

  login:
    image: raresraf/login:latest
    depends_on:
      - mysql
    environment:
      DOCKER_COMPOSE_BUILD: "true"
    ports:
      - "31001:5000"
    networks:
      - backend
      - frontend
    deploy:
      replicas: 1
      placement:
        constraints: [node.role == worker]

  webmonitoringapi:
    image: raresraf/webmonitoringapi:latest
    depends_on:
      - mysql
    environment:
      DOCKER_COMPOSE_BUILD: "true"
    ports:
      - "31002:5000"
    networks:
      - backend
      - frontend
    deploy:
      replicas: 2
      placement:
        constraints: [node.role == worker]

  # docker build -f metricsUI/Dockerfile -t raresraf/metricsui:latest .
  metricsui:
    image: raresraf/metricsui:latest
    depends_on:
      - webmonitoringapi
      - login
    environment:
      DOCKER_COMPOSE_BUILD: "true"
    ports:
      - "31000:80"
    networks:
      - frontend
    deploy:
      replicas: 2
      placement:
        constraints: [node.role == worker]

  visualizer:
    image: dockersamples/visualizer:stable
    ports:
      - "8080:8080"
    stop_grace_period: 1m30s
    volumes:
      - "/var/run/docker.sock:/var/run/docker.sock"
    deploy:
      replicas: 1
      placement:
        constraints: [node.role == manager]

  grafana:
    image: grafana/grafana:latest
    ports:
      - "32001:3000"
    depends_on:
      - mysql
    environment:
      - "GF_SECURITY_ADMIN_USER=TestUsername"
      - "GF_SECURITY_ADMIN_PASSWORD=TestUsername"
    user: "0"
    networks:
      - backend
    deploy:
      replicas: 1
      placement:
        constraints: [node.role == worker]

volumes:
  db-volume:

networks:
  backend:
  frontend:
"""

EXPECTED_DOCKER_COMPOSE = """networks:
  backend: null
  frontend: null
services:
  grafana:
    depends_on:
    - mysql
    deploy:
      replicas: 1
    environment:
    - GF_SECURITY_ADMIN_USER=TestUsername
    - GF_SECURITY_ADMIN_PASSWORD=TestUsername
    image: grafana/grafana:latest
    networks:
    - backend
    ports:
    - 32001:3000
    user: '0'
  login:
    depends_on:
    - mysql
    deploy:
      replicas: 1
    environment:
      DOCKER_COMPOSE_BUILD: 'true'
    image: raresraf/login:latest
    networks:
    - backend
    - frontend
    ports:
    - 31001:5000
  metricsui:
    depends_on:
    - webmonitoringapi
    - login
    deploy:
      replicas: 2
    environment:
      DOCKER_COMPOSE_BUILD: 'true'
    image: raresraf/metricsui:latest
    networks:
    - frontend
    ports:
    - 31000:80
  mysql:
    deploy:
      replicas: 1
    environment:
      DOCKER_COMPOSE_BUILD: 'true'
    image: raresraf/rafmetricsmysql:latest
    networks:
    - backend
    ports:
    - 3306:3306
    volumes:
    - db-volume:/var/lib/mysql
  resourcemonitor:
    depends_on:
    - mysql
    deploy:
      replicas: 1
    environment:
      DOCKER_COMPOSE_BUILD: 'true'
    image: raresraf/resourcemonitor:latest
    networks:
    - backend
  visualizer:
    deploy:
      replicas: 1
    image: dockersamples/visualizer:stable
    ports:
    - 8080:8080
    stop_grace_period: 1m30s
    volumes:
    - /var/run/docker.sock:/var/run/docker.sock
  webmonitoringapi:
    depends_on:
    - mysql
    deploy:
      replicas: 2
    environment:
      DOCKER_COMPOSE_BUILD: 'true'
    image: raresraf/webmonitoringapi:latest
    networks:
    - backend
    - frontend
    ports:
    - 31002:5000
  websitemonitor:
    depends_on:
    - mysql
    deploy:
      replicas: 1
    environment:
      DOCKER_COMPOSE_BUILD: 'true'
    image: raresraf/websitemonitor:latest
    networks:
    - backend
version: '3.3'
volumes:
  db-volume: null
"""



import pymysql
from flask import Flask
from flask import jsonify
from flask import request
from flask_cors import CORS
from flaskext.mysql import MySQL
from werkzeug.security import generate_password_hash, check_password_hash

from Login.settings import (
    MYSQL_DATABASE_USER,
    MYSQL_DATABASE_PASSWORD,
    MYSQL_DATABASE_DB,
    MYSQL_DATABASE_HOST,
)

app = Flask(__name__)
CORS(app)

mysql = MySQL()

# MySQL configurations
app.config["MYSQL_DATABASE_USER"] = MYSQL_DATABASE_USER
app.config["MYSQL_DATABASE_PASSWORD"] = MYSQL_DATABASE_PASSWORD
app.config["MYSQL_DATABASE_DB"] = MYSQL_DATABASE_DB
app.config["MYSQL_DATABASE_HOST"] = MYSQL_DATABASE_HOST

# Workaround for KeyError: MYSQL_DATABASE_SOCKET
app.config["MYSQL_DATABASE_SOCKET"] = None

mysql.init_app(app)


@app.route("/")
def index():
    return "Hello, world!"


@app.route("/add", methods=["POST"])
def add_user():
    try:
        _json = request.json
        _last_name = _json["last_name"]
        _first_name = _json["first_name"]
        _username = _json["username"]
        _email = _json["email"]
        _password = _json["pwd"]
        # Validate the received values
        if (_last_name and _first_name and _username and _email and _password
                and request.method == "POST"):
            # Get password hash
            _hashed_password = generate_password_hash(_password)
            # Save edits
            sql = "INSERT INTO USERS(LastName, FirstName, Username, Email, hashedpassword) VALUES(%s, %s, %s, %s, %s)"
            data = (_last_name, _first_name, _username, _email,
                    _hashed_password)
            conn = mysql.connect()
            cursor = conn.cursor()
            cursor.execute(sql, data)
            conn.commit()
            resp = jsonify("User added successfully!")
            resp.status_code = 200
            return resp
        else:
            return not_found()
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.route("/users")
def users():
    try:
        conn = mysql.connect()
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute("SELECT * FROM USERS")
        rows = cursor.fetchall()
        resp = jsonify(rows)
        resp.status_code = 200
        return resp
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.route("/user/<id>")
def user(id):
    try:
        conn = mysql.connect()
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute("SELECT * FROM USERS WHERE Username=%s", id)
        row = cursor.fetchone()
        resp = jsonify(row)
        resp.status_code = 200
        return resp
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


def get_user_id(id):
    try:
        conn = mysql.connect()
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute("SELECT * FROM USERS WHERE Username=%s", id)
        row = cursor.fetchone()
        return row
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.route("/user/<id>/<passwd>")
def auth_user(id, passwd):
    try:
        row = get_user_id(id)
        ref_passwd = ""
        if row:
            ref_passwd = row.get("hashedpassword", "")
        if check_password_hash(ref_passwd, passwd):
            resp = jsonify(authenticated=True)
            resp.status_code = 200
            return resp
        else:
            resp = jsonify(authenticated=False)
            resp.status_code = 200
            return resp
    except Exception as e:
        print(e)


@app.route("/update/<id>", methods=["POST"])
def update_user(id):
    try:
        _json = request.json
        _last_name = _json["last_name"]
        _first_name = _json["first_name"]
        _email = _json["email"]
        _password = _json["pwd"]
        # Validate the received values
        if (_last_name and _first_name and _email and _password
                and request.method == "POST"):
            # Get password hash
            _hashed_password = generate_password_hash(_password)
            # Save edits
            sql = "UPDATE USERS SET LastName=%s, FirstName=%s, Email=%s, hashedpassword=%s WHERE Username=%s"
            data = (_last_name, _first_name, _email, _hashed_password, id)
            conn = mysql.connect()
            cursor = conn.cursor()
            cursor.execute(sql, data)
            conn.commit()
            resp = jsonify("User updated successfully!")
            resp.status_code = 200
            return resp
        else:
            return not_found()
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.route("/delete/<id>")
def delete_user(id):
    try:
        conn = mysql.connect()
        cursor = conn.cursor()
        cursor.execute("DELETE FROM USERS WHERE Username=%s", (id, ))
        conn.commit()
        resp = jsonify("User deleted successfully!")
        resp.status_code = 200
        return resp
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp


@app.errorhandler(404)
def not_found(error=None):
    message = {"status": 404, "message": "Not Found: " + request.url}
    resp = jsonify(message)
    resp.status_code = 404

    return resp


if __name__ == "__main__":
    app.run(host="0.0.0.0", debug=True)



import numpy as np

DISCOUNT_FACTOR = 16


def generate_larger_evaluation_set(x):
    PREDICTION_RANGE = int(np.size(x) / DISCOUNT_FACTOR)

    latest_element = np.max(x)
    latest_element2 = np.partition(x.flatten(), -2)[-2]
    diff = DISCOUNT_FACTOR * (latest_element - latest_element2)

    extra_x = np.array([])
    for it in range(0, PREDICTION_RANGE + 1):
        extra_x = np.append(extra_x, latest_element + diff * it)

    return extra_x



from flask import jsonify

from Login.app import get_user_id
from Login.app import not_found


def delete_resource_wrapper(mysql, request):
    try:
        _json = request.json
        _username = _json["username"]
        _resource = _json["resource"]
        _command = _json["command"]
        # validate the received values
        if _username and _resource and _command and request.method == "POST":
            # save edits
            get_user_info = get_user_id(_username)
            if not get_user_info:
                return not_found()
            _userid = get_user_info.get("Userid", 1)
            sql = "DELETE FROM RESOURCE where Userid = %s and ResourceName = %s and Command = %s"
            data = (_userid, _resource, _command)
            conn = mysql.connect()
            cursor = conn.cursor()
            cursor.execute(sql, data)
            conn.commit()
            resp = jsonify("Resource deleted successfully!")
            resp.status_code = 200
            return resp
        else:
            return not_found()
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp



from pprint import pprint

import gym

env = gym.make('ChessVsRandomBot-v0')


def available_moves():
    state = env.state
    moves_p1 = env.get_possible_moves(state, 1)
    moves_p2 = env.get_possible_moves(state, -1)
    pprint(moves_p1)
    pprint(moves_p2)

    # no actions left -> resign
    if len(moves_p1) == 0:
        print('resigning is the only move...')
        resign_action = env.resign()

    # chess coordinates Player 1
    for m in moves_p1:
        print(env.convert_coords(m))

    # chess coordinates Player 2
    for m in moves_p2:
        print(env.convert_coords(m))

    # Player 1 moves
    for piece in set([m['piece_id'] for m in moves_p1]):
        env.render_moves(state, piece, moves_p1, mode='human')

    # Player 2 moves
    for piece in set([m['piece_id'] for m in moves_p2]):
        env.render_moves(state, piece, moves_p2, mode='human')


if __name__ == "__main__":
    available_moves()



import pymysql


def get_last_seven_entries_time(mysql, resource_name):
    try:
        conn = mysql.connect()
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute(
            "select ResponseTime from PING where Resourceid = %s order by Timestamp desc limit 7;"
            % (resource_name))
        rows = cursor.fetchall()
        return rows
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return None


def get_last_seven_entries_size(mysql, resource_name):
    try:
        conn = mysql.connect()
        cursor = conn.cursor(pymysql.cursors.DictCursor)
        cursor.execute(
            "select ResponseSize from PING where Resourceid = %s order by Timestamp desc limit 7;"
            % (resource_name))
        rows = cursor.fetchall()
        return rows
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return None



import requests
import time

from WebMonitoring.configs.constants import MonitoringStatus, RequestTypes, sample_url
from WebMonitoring.configs.settings import SHOW_PROGRESS_MESSAGE


class ResourceMonitor:
    """Executes simple requests on a given resource. Measure total time for request response"""
    def __init__(self,
                 url,
                 timeout=10,
                 requestType=RequestTypes.GET,
                 requestPayload=None):
        self.url = url
        self.timeout = timeout
        self.status = MonitoringStatus.WAITING
        self.requestType = requestType
        self.requestPayload = requestPayload

        # Metrics returned
        self.responseTime = 0
        self.responseSize = 0
        self.timestamp = time.localtime()

    def execute_request(self):
        self.timestamp = time.localtime()
        if self.requestType == RequestTypes.GET:
            return requests.get(self.url,
                                timeout=self.timeout,
                                params=self.requestPayload)
        if self.requestType == RequestTypes.POST:
            return requests.post(self.url,
                                 timeout=self.timeout,
                                 data=self.requestPayload)

    def update_metrics(self, responseTime, responseSize):
        self.responseTime = responseTime
        self.responseSize = responseSize

    def get_metrics(self):
        """Return a tuple (responseTime, responseSize)"""
        return (self.responseTime, self.responseSize)

    def get_timestamp(self):
        """Return the timestamp corresponding to the execution time of the command"""
        return self.timestamp

    def run(self):
        try:
            self.status = MonitoringStatus.START
            r = self.execute_request()

            self.status = MonitoringStatus.PROGRESS
            elapsed_seconds = r.elapsed.total_seconds()
            size = len(r.content)
            self.update_metrics(elapsed_seconds, size)

            self.status = MonitoringStatus.DONE

        except requests.exceptions.ConnectTimeout:
            self.status = MonitoringStatus.FAILED
            self.update_metrics(-1, 0)

        except requests.exceptions.ConnectionError:
            self.status = MonitoringStatus.FAILED
            self.update_metrics(-1, 0)

        except Exception:
            self.status = MonitoringStatus.FAILED
            self.update_metrics(-1, 0)

        self.verbose()

    def verbose(self):
        if SHOW_PROGRESS_MESSAGE:
            print("%s request status: %s" % (self.requestType, self.status))
            print("URL: %s" % self.url)
            if self.status == MonitoringStatus.DONE:
                print("Request total elapsed time: %.3f s" % self.responseTime)
                print("Request size: %.2f KBytes" %
                      (self.responseSize / 1024.0))


# Sample driver for ResourceMonitor class
class DriverResourceMonitor:
    def __init__(self):
        self.sample_url = sample_url

    def run(self):
        monitor = ResourceMonitor(sample_url)
        monitor.run()
        print(monitor.get_metrics())



EXPECTED_DAILY_WEBSITE_GENERATE_SAMPLES_QUERIES = """delimiter //
DROP PROCEDURE IF EXISTS get_daily_samples_websites;
CREATE PROCEDURE get_daily_samples_websites (
    IN id INT,
    OUT entry0 FLOAT,
    OUT entry1 FLOAT,
    OUT entry2 FLOAT,
    OUT entry3 FLOAT,
    OUT entry4 FLOAT,
    OUT entry5 FLOAT,
    OUT entry6 FLOAT,
    OUT entry7 FLOAT,
    OUT entry8 FLOAT,
    OUT entry9 FLOAT,
    OUT entry10 FLOAT,
    OUT entry11 FLOAT,
    OUT entry12 FLOAT,
    OUT entry13 FLOAT,
    OUT entry14 FLOAT,
    OUT entry15 FLOAT,
    OUT entry16 FLOAT,
    OUT entry17 FLOAT,
    OUT entry18 FLOAT,
    OUT entry19 FLOAT,
    OUT entry20 FLOAT,
    OUT entry21 FLOAT,
    OUT entry22 FLOAT,
    OUT entry23 FLOAT,
    OUT start_hour FLOAT
 )
BEGIN
    select HOUR(now()) INTO start_hour;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 24 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 23 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry0 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 24 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 23 HOUR) AND Websiteid = id limit 1;
        else SET entry0 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 23 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 22 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry1 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 23 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 22 HOUR) AND Websiteid = id limit 1;
        else SET entry1 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 22 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 21 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry2 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 22 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 21 HOUR) AND Websiteid = id limit 1;
        else SET entry2 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 21 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 20 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry3 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 21 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 20 HOUR) AND Websiteid = id limit 1;
        else SET entry3 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 20 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 19 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry4 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 20 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 19 HOUR) AND Websiteid = id limit 1;
        else SET entry4 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 19 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 18 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry5 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 19 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 18 HOUR) AND Websiteid = id limit 1;
        else SET entry5 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 18 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 17 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry6 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 18 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 17 HOUR) AND Websiteid = id limit 1;
        else SET entry6 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 17 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 16 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry7 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 17 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 16 HOUR) AND Websiteid = id limit 1;
        else SET entry7 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 16 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 15 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry8 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 16 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 15 HOUR) AND Websiteid = id limit 1;
        else SET entry8 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 15 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 14 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry9 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 15 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 14 HOUR) AND Websiteid = id limit 1;
        else SET entry9 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 14 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 13 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry10 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 14 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 13 HOUR) AND Websiteid = id limit 1;
        else SET entry10 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 13 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 12 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry11 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 13 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 12 HOUR) AND Websiteid = id limit 1;
        else SET entry11 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 12 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 11 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry12 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 12 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 11 HOUR) AND Websiteid = id limit 1;
        else SET entry12 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 11 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 10 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry13 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 11 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 10 HOUR) AND Websiteid = id limit 1;
        else SET entry13 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 10 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 9 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry14 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 10 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 9 HOUR) AND Websiteid = id limit 1;
        else SET entry14 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 9 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 8 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry15 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 9 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 8 HOUR) AND Websiteid = id limit 1;
        else SET entry15 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 8 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 7 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry16 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 8 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 7 HOUR) AND Websiteid = id limit 1;
        else SET entry16 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 7 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 6 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry17 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 7 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 6 HOUR) AND Websiteid = id limit 1;
        else SET entry17 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 6 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 5 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry18 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 6 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 5 HOUR) AND Websiteid = id limit 1;
        else SET entry18 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 5 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 4 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry19 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 5 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 4 HOUR) AND Websiteid = id limit 1;
        else SET entry19 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 4 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 3 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry20 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 4 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 3 HOUR) AND Websiteid = id limit 1;
        else SET entry20 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 3 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 2 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry21 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 3 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 2 HOUR) AND Websiteid = id limit 1;
        else SET entry21 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 2 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 1 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry22 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 2 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 1 HOUR) AND Websiteid = id limit 1;
        else SET entry22 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 1 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 0 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry23 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 1 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 0 HOUR) AND Websiteid = id limit 1;
        else SET entry23 := 0;
    end if;
END//
delimiter ;
"""
EXPECTED_WEEKLY_WEBSITE_GENERATE_SAMPLES_QUERIES = """delimiter //
DROP PROCEDURE IF EXISTS get_weekly_samples_websites;
CREATE PROCEDURE get_weekly_samples_websites (
    IN id INT,
    OUT entry0 FLOAT,
    OUT entry1 FLOAT,
    OUT entry2 FLOAT,
    OUT entry3 FLOAT,
    OUT entry4 FLOAT,
    OUT entry5 FLOAT,
    OUT entry6 FLOAT,
    OUT entry7 FLOAT,
    OUT entry8 FLOAT,
    OUT entry9 FLOAT,
    OUT entry10 FLOAT,
    OUT entry11 FLOAT,
    OUT entry12 FLOAT,
    OUT entry13 FLOAT,
    OUT entry14 FLOAT,
    OUT entry15 FLOAT,
    OUT entry16 FLOAT,
    OUT entry17 FLOAT,
    OUT entry18 FLOAT,
    OUT entry19 FLOAT,
    OUT entry20 FLOAT,
    OUT entry21 FLOAT,
    OUT entry22 FLOAT,
    OUT entry23 FLOAT,
    OUT entry24 FLOAT,
    OUT entry25 FLOAT,
    OUT entry26 FLOAT,
    OUT entry27 FLOAT,
    OUT start_hour FLOAT
 )
BEGIN
    select HOUR(now()) INTO start_hour;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 168 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 162 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry0 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 168 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 162 HOUR) AND Websiteid = id limit 1;
        else SET entry0 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 162 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 156 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry1 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 162 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 156 HOUR) AND Websiteid = id limit 1;
        else SET entry1 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 156 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 150 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry2 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 156 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 150 HOUR) AND Websiteid = id limit 1;
        else SET entry2 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 150 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 144 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry3 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 150 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 144 HOUR) AND Websiteid = id limit 1;
        else SET entry3 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 144 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 138 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry4 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 144 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 138 HOUR) AND Websiteid = id limit 1;
        else SET entry4 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 138 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 132 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry5 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 138 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 132 HOUR) AND Websiteid = id limit 1;
        else SET entry5 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 132 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 126 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry6 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 132 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 126 HOUR) AND Websiteid = id limit 1;
        else SET entry6 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 126 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 120 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry7 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 126 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 120 HOUR) AND Websiteid = id limit 1;
        else SET entry7 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 120 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 114 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry8 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 120 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 114 HOUR) AND Websiteid = id limit 1;
        else SET entry8 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 114 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 108 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry9 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 114 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 108 HOUR) AND Websiteid = id limit 1;
        else SET entry9 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 108 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 102 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry10 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 108 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 102 HOUR) AND Websiteid = id limit 1;
        else SET entry10 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 102 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 96 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry11 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 102 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 96 HOUR) AND Websiteid = id limit 1;
        else SET entry11 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 96 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 90 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry12 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 96 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 90 HOUR) AND Websiteid = id limit 1;
        else SET entry12 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 90 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 84 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry13 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 90 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 84 HOUR) AND Websiteid = id limit 1;
        else SET entry13 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 84 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 78 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry14 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 84 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 78 HOUR) AND Websiteid = id limit 1;
        else SET entry14 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 78 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 72 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry15 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 78 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 72 HOUR) AND Websiteid = id limit 1;
        else SET entry15 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 72 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 66 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry16 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 72 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 66 HOUR) AND Websiteid = id limit 1;
        else SET entry16 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 66 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 60 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry17 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 66 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 60 HOUR) AND Websiteid = id limit 1;
        else SET entry17 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 60 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 54 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry18 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 60 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 54 HOUR) AND Websiteid = id limit 1;
        else SET entry18 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 54 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 48 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry19 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 54 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 48 HOUR) AND Websiteid = id limit 1;
        else SET entry19 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 48 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 42 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry20 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 48 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 42 HOUR) AND Websiteid = id limit 1;
        else SET entry20 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 42 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 36 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry21 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 42 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 36 HOUR) AND Websiteid = id limit 1;
        else SET entry21 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 36 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 30 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry22 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 36 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 30 HOUR) AND Websiteid = id limit 1;
        else SET entry22 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 30 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 24 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry23 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 30 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 24 HOUR) AND Websiteid = id limit 1;
        else SET entry23 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 24 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 18 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry24 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 24 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 18 HOUR) AND Websiteid = id limit 1;
        else SET entry24 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 18 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 12 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry25 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 18 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 12 HOUR) AND Websiteid = id limit 1;
        else SET entry25 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 12 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 6 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry26 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 12 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 6 HOUR) AND Websiteid = id limit 1;
        else SET entry26 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 6 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 0 HOUR) AND Websiteid = id)
        then SELECT TotalTime INTO entry27 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 6 HOUR) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 0 HOUR) AND Websiteid = id limit 1;
        else SET entry27 := 0;
    end if;
END//
delimiter ;
"""
EXPECTED_MONTHLY_WEBSITE_GENERATE_SAMPLES_QUERIES = """delimiter //
DROP PROCEDURE IF EXISTS get_monthly_samples_websites;
CREATE PROCEDURE get_monthly_samples_websites (
    IN id INT,
    OUT entry0 FLOAT,
    OUT entry1 FLOAT,
    OUT entry2 FLOAT,
    OUT entry3 FLOAT,
    OUT entry4 FLOAT,
    OUT entry5 FLOAT,
    OUT entry6 FLOAT,
    OUT entry7 FLOAT,
    OUT entry8 FLOAT,
    OUT entry9 FLOAT,
    OUT entry10 FLOAT,
    OUT entry11 FLOAT,
    OUT entry12 FLOAT,
    OUT entry13 FLOAT,
    OUT entry14 FLOAT,
    OUT entry15 FLOAT,
    OUT entry16 FLOAT,
    OUT entry17 FLOAT,
    OUT entry18 FLOAT,
    OUT entry19 FLOAT,
    OUT entry20 FLOAT,
    OUT entry21 FLOAT,
    OUT entry22 FLOAT,
    OUT entry23 FLOAT,
    OUT entry24 FLOAT,
    OUT entry25 FLOAT,
    OUT entry26 FLOAT,
    OUT entry27 FLOAT,
    OUT entry28 FLOAT,
    OUT entry29 FLOAT,
    OUT entry30 FLOAT,
    OUT start_hour FLOAT
 )
BEGIN
    select DAY(now()) INTO start_hour;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 31 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 30 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry0 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 31 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 30 DAY) AND Websiteid = id limit 1;
        else SET entry0 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 30 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 29 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry1 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 30 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 29 DAY) AND Websiteid = id limit 1;
        else SET entry1 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 29 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 28 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry2 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 29 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 28 DAY) AND Websiteid = id limit 1;
        else SET entry2 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 28 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 27 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry3 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 28 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 27 DAY) AND Websiteid = id limit 1;
        else SET entry3 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 27 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 26 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry4 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 27 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 26 DAY) AND Websiteid = id limit 1;
        else SET entry4 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 26 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 25 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry5 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 26 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 25 DAY) AND Websiteid = id limit 1;
        else SET entry5 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 25 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 24 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry6 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 25 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 24 DAY) AND Websiteid = id limit 1;
        else SET entry6 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 24 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 23 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry7 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 24 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 23 DAY) AND Websiteid = id limit 1;
        else SET entry7 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 23 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 22 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry8 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 23 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 22 DAY) AND Websiteid = id limit 1;
        else SET entry8 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 22 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 21 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry9 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 22 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 21 DAY) AND Websiteid = id limit 1;
        else SET entry9 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 21 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 20 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry10 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 21 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 20 DAY) AND Websiteid = id limit 1;
        else SET entry10 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 20 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 19 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry11 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 20 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 19 DAY) AND Websiteid = id limit 1;
        else SET entry11 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 19 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 18 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry12 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 19 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 18 DAY) AND Websiteid = id limit 1;
        else SET entry12 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 18 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 17 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry13 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 18 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 17 DAY) AND Websiteid = id limit 1;
        else SET entry13 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 17 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 16 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry14 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 17 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 16 DAY) AND Websiteid = id limit 1;
        else SET entry14 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 16 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 15 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry15 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 16 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 15 DAY) AND Websiteid = id limit 1;
        else SET entry15 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 15 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 14 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry16 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 15 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 14 DAY) AND Websiteid = id limit 1;
        else SET entry16 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 14 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 13 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry17 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 14 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 13 DAY) AND Websiteid = id limit 1;
        else SET entry17 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 13 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 12 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry18 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 13 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 12 DAY) AND Websiteid = id limit 1;
        else SET entry18 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 12 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 11 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry19 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 12 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 11 DAY) AND Websiteid = id limit 1;
        else SET entry19 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 11 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 10 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry20 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 11 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 10 DAY) AND Websiteid = id limit 1;
        else SET entry20 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 10 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 9 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry21 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 10 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 9 DAY) AND Websiteid = id limit 1;
        else SET entry21 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 9 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 8 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry22 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 9 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 8 DAY) AND Websiteid = id limit 1;
        else SET entry22 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 8 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 7 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry23 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 8 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 7 DAY) AND Websiteid = id limit 1;
        else SET entry23 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 7 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 6 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry24 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 7 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 6 DAY) AND Websiteid = id limit 1;
        else SET entry24 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 6 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 5 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry25 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 6 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 5 DAY) AND Websiteid = id limit 1;
        else SET entry25 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 5 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 4 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry26 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 5 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 4 DAY) AND Websiteid = id limit 1;
        else SET entry26 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 4 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 3 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry27 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 4 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 3 DAY) AND Websiteid = id limit 1;
        else SET entry27 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 3 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 2 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry28 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 3 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 2 DAY) AND Websiteid = id limit 1;
        else SET entry28 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 2 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 1 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry29 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 2 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 1 DAY) AND Websiteid = id limit 1;
        else SET entry29 := 0;
    end if;
    if EXISTS(SELECT TotalTime FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 1 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 0 DAY) AND Websiteid = id)
        then SELECT TotalTime INTO entry30 FROM WEBSITES_METRICS WHERE TIMESTAMP >= DATE_SUB(NOW(), INTERVAL 1 DAY) AND TIMESTAMP <= DATE_SUB(NOW(), INTERVAL 0 DAY) AND Websiteid = id limit 1;
        else SET entry30 := 0;
    end if;
END//
delimiter ;
"""



from flask import jsonify

from Login.app import get_user_id
from Login.app import not_found


def delete_website_wrapper(mysql, request):
    try:
        _json = request.json
        _username = _json["username"]
        _website_name = _json["website_name"]
        _url = _json["url"]
        # validate the received values
        if _username and _website_name and _url and request.method == "POST":
            # save edits
            get_user_info = get_user_id(_username)
            if not get_user_info:
                return not_found()
            _userid = get_user_info.get("Userid", 1)
            sql = "DELETE FROM WEBSITES where Userid = %s and WebsiteName = %s and WebsiteUrl = %s"
            data = (_userid, _website_name, _url)
            conn = mysql.connect()
            cursor = conn.cursor()
            cursor.execute(sql, data)
            conn.commit()
            resp = jsonify("Website deleted successfully!")
            resp.status_code = 200
            return resp
        else:
            return not_found()
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            resp = jsonify("Operation failed")
            resp.status_code = 404
            return resp



import numpy as np
import sys

from rafComputing.ML.CustomSettings.settings import MAX_ITER
from rafComputing.ML.features.feature_types import POWER_FEATURE_TYPE
from rafComputing.ML.snippets.demo.strassen.CustomStrassenLinearRegressionTrainingMoreGraphsPrediction import \
    CustomStrassenLinearRegressionTrainingMoreGraphsPrediction
from rafComputing.ML.snippets.demo.strassen.ML_strassen_more_graphs import get_feature_val_from_path


def strassen_generate_larger_evaluation_set(x):
    latest_element = np.max(x)
    extra_x = np.linspace(latest_element, 1e8, num=100)
    return extra_x


if __name__ == "__main__":
    if len(sys.argv) <= 1:
        print(
            "Usage v1: python3 rafComputing/ML/snippets/demo/strassen/ML_strassen_more_graphs_prediction.py <file_name1> <file_name2> <file_name3> ... "
        )
        print(
            "Usage (e.g.): python3 rafComputing/ML/snippets/demo/strassen/ML_strassen_more_graphs_prediction.py rComplexity/samples/matrix_multiplication/results/sprmcrogpu-wn13/result_n3_20200417113226 rComplexity/samples/matrix_multiplication/results/sprmcrogpu-wn13/result_n28_20200417113226"
        )
        sys.exit(-1)

    plt_legend = [
        "Regression line(Prediction) \nRegression line: Cache-friendly loop ordering",
        "Regression line(Prediction) \nRegression line: Strassen algorithm",
        "Generated data:  Cache-friendly loop ordering",
        "Generated data:  Strassen algorithm",
    ]

    counter = 0
    for path in sys.argv:
        if path == sys.argv[0]:
            continue
        CustomStrassenLinearRegressionTrainingMoreGraphsPrediction(
            path=path,
            alpha=1e-19,
            n_iterations=MAX_ITER,
            internal_counter=counter,
            feature_type=POWER_FEATURE_TYPE,
            feature_val=get_feature_val_from_path(path),
            first=(path == sys.argv[1]),
            final=(path == sys.argv[len(sys.argv) - 1]),
            plt_legend=plt_legend,
            custom_generate_larger_evaluation_set=
            strassen_generate_larger_evaluation_set)
        counter = counter + 1



user1 = {
    "Created":
    "Thu, 07 Nov 2019 08:46:20 GMT",
    "Email":
    "test@email.com",
    "FirstName":
    "test_first_name",
    "LastName":
    "test_last_name",
    "Userid":
    1,
    "Username":
    "TestUsername",
    "hashedpassword":
    "pbkdf2:sha256:150000$3LCL1PDp$dd4ad95d664ce4b6fc0b7c9fe5870e793c3d364160c83fbd67fb1589fc5ab486",
}

user2 = {
    "Created":
    "Tue, 12 Nov 2019 12:52:59 GMT",
    "Email":
    "flask_test_email",
    "FirstName":
    "flask_test_first_name",
    "LastName":
    "flask_test_last_name",
    "Userid":
    2,
    "Username":
    "flask_test_username",
    "hashedpassword":
    "pbkdf2:sha256:150000$ZIYIaFBX$963996e4c5dbd588b4f24f7a08d4bf801fc0c495e22c779ae333a2901e3b1bd3",
}

users_dict = [user1, user2]



# Settings for LinearRegressionTraining
MAX_ITER = 10000
ALPHA = 1e-25

# MLdriver configs
DEFAULT_ITER_INCREASE_STEPS_LOG = 0
DEFAULT_ITER_INCREASE_STEPS = 2**DEFAULT_ITER_INCREASE_STEPS_LOG
DEFAULT_RANGE_ = range(1, 2)

# LinearRegressionTraining DEFAULT settings
ALPHA_LinearRegressionTraining = 0.1e-20
MAX_ITER_LinearRegressionTraining = 10000

# Output prefix
OUTPUT_PREFIX = "result_"
OUTPUT_PREFIX_0 = "result_0"

# AutoDriver
AUTO_DRIVER_DEFAULT_OUTPUT_NAME = "result.png"



# This is a deprecated script
# Consult README.md

import os

import jsbeautifier

opts = jsbeautifier.default_options()
opts.indent_size = 2
""" Simple script for beautifying JS code """


def jsbeautifier_driver():
    # Set options

    for root, _, files in os.walk("."):
        for file in files:
            if file.endswith(".js"):
                print(os.path.join(root, file))

                full_path = os.path.join(root, file)
                res = jsbeautifier.beautify_file(full_path, opts)

                with open(full_path, "w") as f:
                    f.write(res)


if __name__ == "__main__":
    jsbeautifier_driver()



import os

# Set SHOW_VERBOSE_MESSAGE on true for multiple logs of the monitoring tools
SHOW_VERBOSE_MESSAGE = False

# Set SHOW_PROGRESS_MESSAGE on true for general logs about current process (used in ResourceMonitoring)
SHOW_PROGRESS_MESSAGE = True

# Default speedprofile script location inside repo
# Original speedprofile code: https://github.com/parasdahal/speedprofile
SPEEDPROFILE_LOCATION = "./WebMonitoring/WebsiteMonitorHelpers/entrypoint.sh"

# Define the sample time between two consecutive monitorings
SAMPLE_TIME = 3600

# JSON output location
json_path = "./output/har.json"

# MySQL DB configs
MYSQL_DATABASE_USER = "root"
MYSQL_DATABASE_PASSWORD = "password"
MYSQL_DATABASE_DB = "WebMonitoring"
if os.environ.get("DOCKER_COMPOSE_BUILD"):
    MYSQL_DATABASE_HOST = "mysql"
else:
    MYSQL_DATABASE_HOST = "10.96.0.2"



class MonitoringStatus:
    WAITING = "waiting"
    START = "start"
    QUEUE = "queue"
    PROGRESS = "progress"
    DONE = "done"
    FAILED = "failed"


class DockerResults:
    FAILED = "failed"
    SUCCESS = "success"
    STARTED = "started"
    FINISHED = "finished"


class RequestTypes:
    GET = "GET"
    POST = "POST"


class ResourceEntry:
    def __init__(
        self,
        resource_url,
        resource_name,
        resource_command=RequestTypes.GET,
        resource_payload=None,
    ):
        self.resource_url = resource_url
        self.resource_name = resource_name
        self.resource_command = resource_command
        self.resource_payload = resource_payload


sample_url = "https://github.com/raresraf/rafMetrics/projects/1"
sample_name = "GitHub"
sample_resource_command = RequestTypes.GET
sample_resource_payload = None

sample_resource = ResourceEntry(
    sample_url,
    sample_name,
    resource_command=sample_resource_command,
    resource_payload=sample_resource_payload,
)

resources = [sample_resource]



import random

import gym

env = gym.make('ChessVsRandomBot-v0')

#
# Play against random bot
#
num_episodes = 10
num_steps_per_episode = 50

collected_rewards = []

for i in range(num_episodes):
    initial_state = env.reset()
    print('\n' * 2, '<' * 5, '=' * 10, 'NEW GAME', '=' * 10, '>' * 5)
    # env._render()
    # print('<'*5, '-'*10, 'STARTING', '-'*10, '>'*5)

    player = 1
    total_reward = 0
    done = False
    on_move = 1

    for j in range(num_steps_per_episode):
        state = env.state
        board = state['board']
        kr_moves = state['kr_moves']
        captured = state['captured']

        if done:
            print('>' * 10, 'TOTAL GAME ', i, 'REWARD =', total_reward)
            break

        moves = env.get_possible_moves(state, player)
        # moves = [m for m in moves if m['type'] == 'castling']
        # print(moves)

        if len(moves) == 0:
            a = env.resign_action()
            print('@' * 15, 'PLAYER RESIGNED', '@' * 15)
        else:
            m = random.choice(moves)
            a = env.move_to_actions(m)
            # print('{:6s}'.format(env.convert_coords(m)), end=' ')

        # perform action
        state, reward, done, __ = env.step(a)
        total_reward += reward

    collected_rewards.append(total_reward)

print('\n')
print('#' * 40)
print('#' * 40)
print('#' * 40)
print("\nAVERAGE SCORE: ", sum(collected_rewards) / num_episodes)



import calendar
import datetime

from WebMonitoring.API.constants import PERIOD


def get_timestamp_query(period, i, start):
    if period.lower() == PERIOD.DAILY:
        return (start + i) % 24
    if period.lower() == PERIOD.WEEKLY:
        return (start + 6 * i) % 24
    if period.lower() == PERIOD.MONTHLY:
        today = datetime.date.today()
        first = today.replace(day=1)
        lastMonth = first - datetime.timedelta(days=1)
        modulo = calendar.monthrange(lastMonth.year, lastMonth.month)[1]
        return (start + i - 1) % modulo + 1



# find . -name *.sql | xargs printf "\"%s\", \n"
# Reorder file order to contain ./mysql_config/Tables/* first

SQL_INIT_FILES = [
    "./mysql_config/Tables/create-tables.sql",
    "./mysql_config/Tables/audit_users_table.sql",
    "./mysql_config/Mocks/insert-values.sql",
    "./mysql_config/WebMonitoring/procedures/resource_get_time.sql",
    "./mysql_config/WebMonitoring/procedures/resource_get_size.sql",
    "./mysql_config/WebMonitoring/procedures/resource_get_old_size.sql",
    "./mysql_config/WebMonitoring/procedures/resource_get_old_time.sql",
    "./mysql_config/WebMonitoring/procedures/samples_time_websites/get_monthly_samples_websites.sql",
    "./mysql_config/WebMonitoring/procedures/samples_time_websites/get_daily_samples_websites.sql",
    "./mysql_config/WebMonitoring/procedures/samples_time_websites/get_weekly_samples_websites.sql",
    "./mysql_config/WebMonitoring/procedures/samples_time_resource/get_daily_samples.sql",
    "./mysql_config/WebMonitoring/procedures/samples_time_resource/get_weekly_samples.sql",
    "./mysql_config/WebMonitoring/procedures/samples_time_resource/get_monthly_samples.sql",
    "./mysql_config/WebMonitoring/procedures/samples_size_resource/get_monthly_samples_size.sql",
    "./mysql_config/WebMonitoring/procedures/samples_size_resource/get_weekly_samples_size.sql",
    "./mysql_config/WebMonitoring/procedures/samples_size_resource/get_daily_samples_size.sql",
    "./mysql_config/WebMonitoring/procedures/samples_size_websites/get_monthly_samples_size_websites.sql",
    "./mysql_config/WebMonitoring/procedures/samples_size_websites/get_daily_samples_size_websites.sql",
    "./mysql_config/WebMonitoring/procedures/samples_size_websites/get_weekly_samples_size_websites.sql",
    "./mysql_config/WebMonitoring/functions/resource_get_availability.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_size/resource_statistics_last_24_size/resource_statistic_average_size_24.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_size/resource_statistics_last_24_size/resource_statistic_standard_deviation_size_24.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_size/resource_statistics_last_24_size/resource_statistic_size_24.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_size/resource_statistics_all_size/resource_statistic_size.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_size/resource_statistics_all_size/resource_statistic_standard_deviation_size.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_size/resource_statistics_all_size/resource_statistic_average_size.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_time/resource_statistics_last_24_time/resource_statistic_requests_time_24.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_time/resource_statistics_last_24_time/resource_statistic_average_time_24.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_time/resource_statistics_last_24_time/resource_statistic_standard_deviation_time_24.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_time/resource_statistics_last_24_time/resource_statistic_time_24.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_time/resource_statistics_all_time/resource_statistic_time.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_time/resource_statistics_all_time/resource_statistic_standard_deviation_time.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_time/resource_statistics_all_time/resource_statistic_requests_time.sql",
    "./mysql_config/WebMonitoring/functions/resource_statistics_time/resource_statistics_all_time/resource_statistic_average_time.sql",
    "./mysql_config/Login/triggers/trigger.sql",
]



import os

import mock
import pymysql
from mock import MagicMock

from Login.app import (
    index,
    add_user,
    app,
    users,
    user,
    get_user_id,
    auth_user,
    update_user,
    delete_user,
)
from Login.settings import (
    MYSQL_DATABASE_USER,
    MYSQL_DATABASE_DB,
    MYSQL_DATABASE_HOST,
    MYSQL_DATABASE_PASSWORD,
)
from Login.tests.mocks import users_dict


def test_index():
    ret_index_msg = index()
    assert ret_index_msg == "Hello, world!"


@mock.patch("Login.app.generate_password_hash")
@mock.patch("Login.app.mysql")
def test_add_user(mocked_mysql_var, mocked_hash):
    with app.test_request_context(
            "/add",
            json={
                "last_name": "flask_test_last_name",
                "first_name": "flask_test_first_name",
                "username": "flask_test_username",
                "email": "flask_test_email",
                "pwd": "flask_test_password",
            },
            method="POST",
    ):

        # Same Hash Gen
        mocked_hash.return_value = "pbkdf2:sha256:150000$uQP6jkPa$a70e2bab07128ebf878ba0e0c6ea03a6f397794b36c794bac2087c944d03b40b"

        # Establish MySQL connection -- mocked
        conn = MagicMock()
        mocked_mysql_var.connect.return_value = conn

        # Get a MySQL cursor -- mocked
        cursor = MagicMock()
        conn.cursor.return_value = cursor
        cursor.execute.return_value = MagicMock()
        conn.commit.return_value = MagicMock()
        cursor.close.return_value = MagicMock()
        conn.close.return_value = MagicMock()

        # Call add_user procedure
        resp = add_user()

        # Check workflow
        mocked_hash.assert_called_once_with("flask_test_password")
        mocked_mysql_var.connect.assert_called_once_with()
        conn.cursor.assert_called_once_with()
        cursor.execute.assert_called_once_with(
            "INSERT INTO USERS(LastName, FirstName, Username, Email, hashedpassword) VALUES(%s, %s, %s, %s, %s)",
            (
                "flask_test_last_name",
                "flask_test_first_name",
                "flask_test_username",
                "flask_test_email",
                "pbkdf2:sha256:150000$uQP6jkPa$a70e2bab07128ebf878ba0e0c6ea03a6f397794b36c794bac2087c944d03b40b",
            ),
        )
        conn.commit.assert_called_once_with()
        assert resp.status_code == 200
        assert resp.json == "User added successfully!"
        cursor.close.assert_called_once_with()
        conn.close.assert_called_once_with()


@mock.patch("Login.app.mysql")
def test_users(mocked_mysql_var):
    with app.test_request_context("/users"):

        # Establish MySQL connection -- mocked
        conn = MagicMock()
        mocked_mysql_var.connect.return_value = conn

        # Get a MySQL cursor -- mocked
        cursor = MagicMock()
        conn.cursor.return_value = cursor
        cursor.execute.return_value = MagicMock()
        cursor.fetchall.return_value = users_dict
        cursor.close.return_value = MagicMock()
        conn.close.return_value = MagicMock()

        # Call users procedure
        resp = users()
        expected_json = users_dict

        # Check workflow
        mocked_mysql_var.connect.assert_called_once_with()
        conn.cursor.assert_called_once_with(pymysql.cursors.DictCursor)
        cursor.execute.assert_called_once_with("SELECT * FROM USERS")
        assert resp.status_code == 200
        assert resp.json == expected_json
        cursor.close.assert_called_once_with()
        conn.close.assert_called_once_with()


@mock.patch("Login.app.mysql")
def test_user(mocked_mysql_var):
    with app.test_request_context("/user/TestUsername"):

        # Establish MySQL connection -- mocked
        conn = MagicMock()
        mocked_mysql_var.connect.return_value = conn

        # Get a MySQL cursor -- mocked
        cursor = MagicMock()
        conn.cursor.return_value = cursor
        cursor.execute.return_value = MagicMock()
        cursor.fetchone.return_value = users_dict[0]
        cursor.close.return_value = MagicMock()
        conn.close.return_value = MagicMock()

        # Call users procedure
        id = "TestUsername"
        resp = user(id)
        expected_json = users_dict[0]

        # Check workflow
        mocked_mysql_var.connect.assert_called_once_with()
        conn.cursor.assert_called_once_with(pymysql.cursors.DictCursor)
        cursor.execute.assert_called_once_with(
            "SELECT * FROM USERS WHERE Username=%s", id)
        assert resp.status_code == 200
        assert resp.json == expected_json
        cursor.close.assert_called_once_with()
        conn.close.assert_called_once_with()


@mock.patch("Login.app.mysql")
def test_user_id(mocked_mysql_var):

    # Establish MySQL connection -- mocked
    conn = MagicMock()
    mocked_mysql_var.connect.return_value = conn

    # Get a MySQL cursor -- mocked
    cursor = MagicMock()
    conn.cursor.return_value = cursor
    cursor.execute.return_value = MagicMock()
    cursor.fetchone.return_value = users_dict[0]
    cursor.close.return_value = MagicMock()
    conn.close.return_value = MagicMock()

    # Call users procedure
    id = "TestUsername"
    resp = get_user_id(id)
    expected_json = users_dict[0]

    # Check workflow
    mocked_mysql_var.connect.assert_called_once_with()
    conn.cursor.assert_called_once_with(pymysql.cursors.DictCursor)
    cursor.execute.assert_called_once_with(
        "SELECT * FROM USERS WHERE Username=%s", id)
    assert resp == expected_json
    cursor.close.assert_called_once_with()
    conn.close.assert_called_once_with()


@mock.patch("Login.app.mysql")
def test_auth_user_valid_password(mocked_mysql_var):
    with app.test_request_context("/user/<id>/<passwd>"):

        # Establish MySQL connection -- mocked
        conn = MagicMock()
        mocked_mysql_var.connect.return_value = conn

        # Get a MySQL cursor -- mocked
        cursor = MagicMock()
        conn.cursor.return_value = cursor
        cursor.execute.return_value = MagicMock()
        cursor.fetchone.return_value = users_dict[0]
        cursor.close.return_value = MagicMock()
        conn.close.return_value = MagicMock()

        # Call users procedure
        id = "TestUsername"
        passwd = "TestUsername"
        resp = auth_user(id, passwd)
        expected_json = {"authenticated": True}

        # Check workflow
        mocked_mysql_var.connect.assert_called_once_with()
        conn.cursor.assert_called_once_with(pymysql.cursors.DictCursor)
        cursor.execute.assert_called_once_with(
            "SELECT * FROM USERS WHERE Username=%s", id)
        assert resp.status_code == 200
        assert resp.json == expected_json
        cursor.close.assert_called_once_with()
        conn.close.assert_called_once_with()


@mock.patch("Login.app.mysql")
def test_auth_user_invalid_password(mocked_mysql_var):
    with app.test_request_context("/user/<id>/<passwd>"):

        # Establish MySQL connection -- mocked
        conn = MagicMock()
        mocked_mysql_var.connect.return_value = conn

        # Get a MySQL cursor -- mocked
        cursor = MagicMock()
        conn.cursor.return_value = cursor
        cursor.execute.return_value = MagicMock()
        cursor.fetchone.return_value = users_dict[0]
        cursor.close.return_value = MagicMock()
        conn.close.return_value = MagicMock()

        # Call users procedure
        id = "TestUsername"
        passwd = "InvalidPasswd"
        resp = auth_user(id, passwd)
        expected_json = {"authenticated": False}

        # Check workflow
        mocked_mysql_var.connect.assert_called_once_with()
        conn.cursor.assert_called_once_with(pymysql.cursors.DictCursor)
        cursor.execute.assert_called_once_with(
            "SELECT * FROM USERS WHERE Username=%s", id)
        assert resp.status_code == 200
        assert resp.json == expected_json
        cursor.close.assert_called_once_with()
        conn.close.assert_called_once_with()


@mock.patch("Login.app.generate_password_hash")
@mock.patch("Login.app.mysql")
def test_update_user(mocked_mysql_var, mocked_hash):
    with app.test_request_context(
            "/update/TestUsername",
            json={
                "last_name": "flask_test_last_name",
                "first_name": "flask_test_first_name",
                "username": "flask_test_username",
                "email": "flask_test_email",
                "pwd": "flask_test_password",
            },
            method="POST",
    ):

        # Same Hash Gen
        mocked_hash.return_value = "pbkdf2:sha256:150000$uQP6jkPa$a70e2bab07128ebf878ba0e0c6ea03a6f397794b36c794bac2087c944d03b40b"

        # Establish MySQL connection -- mocked
        conn = MagicMock()
        mocked_mysql_var.connect.return_value = conn

        # Get a MySQL cursor -- mocked
        cursor = MagicMock()
        conn.cursor.return_value = cursor
        cursor.execute.return_value = MagicMock()
        conn.commit.return_value = MagicMock()
        cursor.close.return_value = MagicMock()
        conn.close.return_value = MagicMock()

        # Call add_user procedure
        id = "TestUsername"
        resp = update_user(id)

        # Check workflow
        mocked_hash.assert_called_once_with("flask_test_password")
        mocked_mysql_var.connect.assert_called_once_with()
        conn.cursor.assert_called_once_with()
        cursor.execute.assert_called_once_with(
            "UPDATE USERS SET LastName=%s, FirstName=%s, Email=%s, hashedpassword=%s WHERE Username=%s",
            (
                "flask_test_last_name",
                "flask_test_first_name",
                "flask_test_email",
                "pbkdf2:sha256:150000$uQP6jkPa$a70e2bab07128ebf878ba0e0c6ea03a6f397794b36c794bac2087c944d03b40b",
                "TestUsername",
            ),
        )
        conn.commit.assert_called_once_with()
        assert resp.status_code == 200
        assert resp.json == "User updated successfully!"
        cursor.close.assert_called_once_with()
        conn.close.assert_called_once_with()


@mock.patch("Login.app.mysql")
def test_delete(mocked_mysql_var, ):
    with app.test_request_context(
            "/update/TestUsername",
            json={
                "last_name": "flask_test_last_name",
                "first_name": "flask_test_first_name",
                "username": "flask_test_username",
                "email": "flask_test_email",
                "pwd": "flask_test_password",
            },
            method="POST",
    ):

        # Establish MySQL connection -- mocked
        conn = MagicMock()
        mocked_mysql_var.connect.return_value = conn

        # Get a MySQL cursor -- mocked
        cursor = MagicMock()
        conn.cursor.return_value = cursor
        cursor.execute.return_value = MagicMock()
        conn.commit.return_value = MagicMock()
        cursor.close.return_value = MagicMock()
        conn.close.return_value = MagicMock()

        # Call add_user procedure
        id = "TestUsername"
        resp = delete_user(id)

        # Check workflow
        mocked_mysql_var.connect.assert_called_once_with()
        conn.cursor.assert_called_once_with()
        cursor.execute.assert_called_once_with(
            "DELETE FROM USERS WHERE Username=%s", (id, ))
        conn.commit.assert_called_once_with()
        assert resp.status_code == 200
        assert resp.json == "User deleted successfully!"
        cursor.close.assert_called_once_with()
        conn.close.assert_called_once_with()


def test_db_configs():
    assert MYSQL_DATABASE_USER == "root"
    assert MYSQL_DATABASE_PASSWORD == "password"
    assert MYSQL_DATABASE_DB == "WebMonitoring"
    if os.environ.get("DOCKER_COMPOSE_BUILD"):
        assert MYSQL_DATABASE_HOST == "mysql"
    else:
        assert MYSQL_DATABASE_HOST == "10.96.0.2"



import yaml

from docker_config.generators.no_constraint_compose import remove_placement_contraints
from docker_config.generators.tests.mocks import (
    SAMPLE_DOCKER_COMPOSE,
    EXPECTED_DOCKER_COMPOSE,
)


def test_remove_placement_contraints():
    dict = yaml.safe_load(SAMPLE_DOCKER_COMPOSE)
    dict = remove_placement_contraints(dict)
    out_dict = yaml.dump(dict, default_flow_style=False, allow_unicode=True)
    assert out_dict == EXPECTED_DOCKER_COMPOSE



import matplotlib.pyplot as plt
import numpy as np
import sys

if __name__ == "__main__":
    #python3 rComplexity/plotting/plot_mesh.py rComplexity/samples/gym-chess-master/results/macOS/PYTHON_chess_results_20200603145240
    if len(sys.argv) != 2:
        print("Usage: python3 plot_mesh.py <file_name>")
        sys.exit(-1)

    path = sys.argv[1]
    matrix = np.loadtxt(path, usecols=range(3))

    X3D = matrix[:, 0]
    Y3D = matrix[:, 1]
    Z = matrix[:, 2].reshape(len(set(X3D)), len(set(Y3D)))

    X3D = X3D.reshape(Z.shape)
    Y3D = Y3D.reshape(Z.shape)

    print(X3D)
    print(Y3D)
    print(Z)

    fig = plt.figure()
    ax = plt.axes(projection='3d')
    ax.plot_surface(X3D,
                    Y3D,
                    Z,
                    rstride=1,
                    cstride=1,
                    cmap='viridis',
                    edgecolor='none')
    ax.set_title("Chess game simulation")
    ax.set_xlabel("num_episodes")
    ax.set_ylabel("num_steps_per_episode")
    ax.set_zlabel("Total time (seconds)")

    plt.show()



# Helpers for rendering metrics


# Wrapper avoiding div by ZERO
def div_0(x, y):
    if not y:
        return 0
    if not x:
        return 0
    xx = float(x)
    yy = float(y)
    if yy == 0:
        return 2147483647
    return xx / yy


# Less than implementation with None < Anything
def lt_w_none(val1, val2):
    if val1 is None:
        return True
    if val2 is None:
        return True
    return val1 > val2


def get_percent_w_none(val1, val2):
    if val1 is None or val1 == 0:
        return 100
    if val2 is None or val2 == 0:
        return 0
    return round(abs(val2 - val1) / val1, 2)



from WebMonitoring.API.constants import PERIOD, BYTES_TO_KBYTES
from WebMonitoring.API.resources.helpers import get_timestamp_query


def resources_get_samples_size_daily(mysql, resource_name):
    try:
        conn = mysql.connect()
        cursor = conn.cursor()
        """
        Compatibility warning: PEP-249 specifies that any modified
        parameters must be returned. This is currently impossible
        as they are only available by storing them in a server
        variable and then retrieved by a query. Since stored
        procedures return zero or more result sets, there is no
        reliable way to get at OUT or INOUT parameters via callproc.
        The server variables are named @_procname_n, where procname
        is the parameter above and n is the position of the parameter
        (from zero). Once all result sets generated by the procedure
        have been fetched, you can issue a SELECT @_procname_0, ...
        query using .execute() to get any OUT or INOUT values.
        """
        args = [
            resource_name,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ]

        cursor.callproc("get_daily_samples_size", args)
        cursor.execute("SELECT "
                       "@_get_daily_samples_size_0, "
                       "@_get_daily_samples_size_1, "
                       "@_get_daily_samples_size_2, "
                       "@_get_daily_samples_size_3, "
                       "@_get_daily_samples_size_4, "
                       "@_get_daily_samples_size_5, "
                       "@_get_daily_samples_size_6, "
                       "@_get_daily_samples_size_7, "
                       "@_get_daily_samples_size_8, "
                       "@_get_daily_samples_size_9, "
                       "@_get_daily_samples_size_10, "
                       "@_get_daily_samples_size_11, "
                       "@_get_daily_samples_size_12, "
                       "@_get_daily_samples_size_13, "
                       "@_get_daily_samples_size_14, "
                       "@_get_daily_samples_size_15, "
                       "@_get_daily_samples_size_16, "
                       "@_get_daily_samples_size_17, "
                       "@_get_daily_samples_size_18, "
                       "@_get_daily_samples_size_19, "
                       "@_get_daily_samples_size_20, "
                       "@_get_daily_samples_size_21, "
                       "@_get_daily_samples_size_22, "
                       "@_get_daily_samples_size_23, "
                       "@_get_daily_samples_size_24, "
                       "@_get_daily_samples_size_25 ")
        result_args_get_time = cursor.fetchone()

        result = []
        for i in range(len(result_args_get_time) - 2):
            result.append({
                "custom_data":
                result_args_get_time[i + 1] / BYTES_TO_KBYTES,
                "label":
                get_timestamp_query(PERIOD.DAILY, i, result_args_get_time[-1]),
            })

        return result
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return None


def resources_get_samples_size_weekly(mysql, resource_name):
    try:
        conn = mysql.connect()
        cursor = conn.cursor()
        """
        Compatibility warning: PEP-249 specifies that any modified
        parameters must be returned. This is currently impossible
        as they are only available by storing them in a server
        variable and then retrieved by a query. Since stored
        procedures return zero or more result sets, there is no
        reliable way to get at OUT or INOUT parameters via callproc.
        The server variables are named @_procname_n, where procname
        is the parameter above and n is the position of the parameter
        (from zero). Once all result sets generated by the procedure
        have been fetched, you can issue a SELECT @_procname_0, ...
        query using .execute() to get any OUT or INOUT values.
        """
        args = [
            resource_name,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ]

        cursor.callproc("get_weekly_samples_size", args)
        cursor.execute("SELECT "
                       "@_get_weekly_samples_size_0, "
                       "@_get_weekly_samples_size_1, "
                       "@_get_weekly_samples_size_2, "
                       "@_get_weekly_samples_size_3, "
                       "@_get_weekly_samples_size_4, "
                       "@_get_weekly_samples_size_5, "
                       "@_get_weekly_samples_size_6, "
                       "@_get_weekly_samples_size_7, "
                       "@_get_weekly_samples_size_8, "
                       "@_get_weekly_samples_size_9, "
                       "@_get_weekly_samples_size_10, "
                       "@_get_weekly_samples_size_11, "
                       "@_get_weekly_samples_size_12, "
                       "@_get_weekly_samples_size_13, "
                       "@_get_weekly_samples_size_14, "
                       "@_get_weekly_samples_size_15, "
                       "@_get_weekly_samples_size_16, "
                       "@_get_weekly_samples_size_17, "
                       "@_get_weekly_samples_size_18, "
                       "@_get_weekly_samples_size_19, "
                       "@_get_weekly_samples_size_20, "
                       "@_get_weekly_samples_size_21, "
                       "@_get_weekly_samples_size_22, "
                       "@_get_weekly_samples_size_23, "
                       "@_get_weekly_samples_size_24, "
                       "@_get_weekly_samples_size_25, "
                       "@_get_weekly_samples_size_26, "
                       "@_get_weekly_samples_size_27, "
                       "@_get_weekly_samples_size_28, "
                       "@_get_weekly_samples_size_29 ")
        result_args_get_time = cursor.fetchone()

        result = []
        for i in range(len(result_args_get_time) - 2):
            result.append({
                "custom_data":
                result_args_get_time[i + 1] / BYTES_TO_KBYTES,
                "label":
                get_timestamp_query(PERIOD.WEEKLY, i,
                                    result_args_get_time[-1]),
            })
        return result
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return None


def resources_get_samples_size_monthly(mysql, resource_name):
    try:
        conn = mysql.connect()
        cursor = conn.cursor()
        """
        Compatibility warning: PEP-249 specifies that any modified
        parameters must be returned. This is currently impossible
        as they are only available by storing them in a server
        variable and then retrieved by a query. Since stored
        procedures return zero or more result sets, there is no
        reliable way to get at OUT or INOUT parameters via callproc.
        The server variables are named @_procname_n, where procname
        is the parameter above and n is the position of the parameter
        (from zero). Once all result sets generated by the procedure
        have been fetched, you can issue a SELECT @_procname_0, ...
        query using .execute() to get any OUT or INOUT values.
        """
        args = [
            resource_name,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
            0,
        ]

        cursor.callproc("get_monthly_samples_size", args)
        cursor.execute("SELECT "
                       "@_get_monthly_samples_size_0, "
                       "@_get_monthly_samples_size_1, "
                       "@_get_monthly_samples_size_2, "
                       "@_get_monthly_samples_size_3, "
                       "@_get_monthly_samples_size_4, "
                       "@_get_monthly_samples_size_5, "
                       "@_get_monthly_samples_size_6, "
                       "@_get_monthly_samples_size_7, "
                       "@_get_monthly_samples_size_8, "
                       "@_get_monthly_samples_size_9, "
                       "@_get_monthly_samples_size_10, "
                       "@_get_monthly_samples_size_11, "
                       "@_get_monthly_samples_size_12, "
                       "@_get_monthly_samples_size_13, "
                       "@_get_monthly_samples_size_14, "
                       "@_get_monthly_samples_size_15, "
                       "@_get_monthly_samples_size_16, "
                       "@_get_monthly_samples_size_17, "
                       "@_get_monthly_samples_size_18, "
                       "@_get_monthly_samples_size_19, "
                       "@_get_monthly_samples_size_20, "
                       "@_get_monthly_samples_size_21, "
                       "@_get_monthly_samples_size_22, "
                       "@_get_monthly_samples_size_23, "
                       "@_get_monthly_samples_size_24, "
                       "@_get_monthly_samples_size_25, "
                       "@_get_monthly_samples_size_26, "
                       "@_get_monthly_samples_size_27, "
                       "@_get_monthly_samples_size_28, "
                       "@_get_monthly_samples_size_29, "
                       "@_get_monthly_samples_size_30, "
                       "@_get_monthly_samples_size_31, "
                       "@_get_monthly_samples_size_32")
        result_args_get_time = cursor.fetchone()

        result = []
        for i in range(len(result_args_get_time) - 2):
            result.append({
                "custom_data":
                result_args_get_time[i + 1] / BYTES_TO_KBYTES,
                "label":
                get_timestamp_query(PERIOD.MONTHLY, i,
                                    result_args_get_time[-1]),
            })
        return result
    except Exception as e:
        print(e)
    finally:
        try:
            cursor.close()
            conn.close()
        except NameError:
            return None



from mysql_config.settings import SQL_INIT_FILES


def generate_init_sql():
    # Generate SQL init file for dockerized deployment

    f = open("./mysql_config/init.sql", "w")

    f.write("# Auto-generated init SQL file.\n")
    f.write("# Do not manually edit this file.\n\n\n")
    f.write("CREATE DATABASE IF NOT EXISTS WebMonitoring;\n\n")
    f.write("use WebMonitoring;\n\n\n")

    for tmpfile_name in SQL_INIT_FILES:
        with open(tmpfile_name, "r") as tmpfile:
            tmpfile_read = tmpfile.read()
            f.write(tmpfile_read)
            f.write("\n")


if __name__ == "__main__":
    generate_init_sql()



import mysql.connector
import time

from WebMonitoring.configs.constants import ResourceEntry, RequestTypes
from WebMonitoring.configs.settings import (
    SAMPLE_TIME,
    MYSQL_DATABASE_USER,
    MYSQL_DATABASE_PASSWORD,
    MYSQL_DATABASE_HOST,
    MYSQL_DATABASE_DB,
)
from WebMonitoring.monitors.ResourceMonitor import ResourceMonitor


class ResourceManager:
    def __init__(self, sample_time=3600):
        self.sample_time = sample_time
        self.resources = []
        self.cnx = None

    def run(self):
        for resource in self.resources:

            monitor = ResourceMonitor(
                resource.resource_url,
                requestType=resource.resource_command,
                requestPayload=resource.resource_payload,
            )
            monitor.run()

            cursor = self.cnx.cursor(buffered=True)
            query = (
                "SELECT Resourceid, ResourceName from RESOURCE where ResourceName = '%s'"
                % resource.resource_url)
            cursor.execute(query)

            for (resource_id, _) in cursor:
                query = (
                    "INSERT INTO PING(Resourceid, ResponseTime, ResponseSize) "
                    "VALUES (%d, %lf, %d)") % (resource_id,
                                               monitor.get_metrics()[0],
                                               monitor.get_metrics()[1])
                cursor_insert = self.cnx.cursor(buffered=True)
                cursor_insert.execute(query)
                self.cnx.commit()

    def start(self):
        # Connect to DB
        self.cnx = mysql.connector.connect(
            user=MYSQL_DATABASE_USER,
            password=MYSQL_DATABASE_PASSWORD,
            host=MYSQL_DATABASE_HOST,
            database=MYSQL_DATABASE_DB,
        )

        while True:
            self.resources = []

            # Check for any update in the list of resources
            cursor = self.cnx.cursor(buffered=True)
            query = "SELECT ResourceName, Command from RESOURCE"
            cursor.execute(query)

            for (resource_name, command) in cursor:
                if command == RequestTypes.GET:
                    self.resources.append(
                        ResourceEntry(resource_name, resource_name,
                                      RequestTypes.GET, None))
                elif command.contains(RequestTypes.POST):
                    self.resources.append(
                        # TODO : Parse payload
                        ResourceEntry(resource_name, resource_name,
                                      RequestTypes.POST, None))

            self.run()
            time.sleep(self.sample_time)


if __name__ == "__main__":
    ResourceManager(SAMPLE_TIME).start()



import numpy as np

from rafComputing.ML.features.feature_types import (
    POLYNOMIAL_FEATURE_TYPE,
    NO_FEATURE_TYPE,
    POWER_FEATURE_TYPE,
)


def extract_features(X=None,
                     feature_type=POLYNOMIAL_FEATURE_TYPE,
                     feature_val=1.0):
    if X is None:
        return None
    if feature_type == POLYNOMIAL_FEATURE_TYPE:
        return extract_polynomial_features(X, feature_val)
    if feature_type == POWER_FEATURE_TYPE:
        return extract_power_features(X, feature_val)
    if feature_type == NO_FEATURE_TYPE:
        return same_features(X, feature_val)


def extract_polynomial_features(X, M):
    M = int(M)
    phi = np.ones((X.size, M + 1))
    for i in range(X.size):
        for j in range(M + 1):
            phi[i][j] = np.power(X[i], j)
    return phi


def extract_power_features(X, M):
    phi = np.ones((X.size, 1))
    for i in range(X.size):
        phi[i][0] = np.power(X[i], M)
    return phi


def same_features(X, _M):
    return X
